#include "../rasperf.h"
#include<stdio.h>
#include<stdlib.h>
#include<arpa/inet.h>
#include<sys/types.h>
#include<sys/socket.h>
#include<netdb.h>
#include<string.h>
//Macro definitions
#define handle_error(msg) \
    do { perror(msg); exit(1);} while(0);
#define BUFFER_SIZE 1024 * 1024
#define STRIDE_SIZE 512
#define BACKLOG 5 //buff size/stride size

//A random string generator of given length
char* generate_string(int length)
{
    //Random number seed
    srand((unsigned int) ctime(0));
    char * buff;
    buff=(char *)malloc(sizeof(char)*(length+1));
    int i;
    for(i=0;i< length;i++){
        buff[i] = rand() % rand() % (126 - 33 + 1) + 33;
    }
    buff[i] = '\0';
    return buff;
}

int main(int argc, char*argv[])
{   
    int socket_fd, rc;
    char buff[BUFFER_SIZE];
    char *msg;
    float delta = 0;
    if(argc<2){
        handle_error("<usage>:tcp_client <IP address>");
    }
    //Retrieve the protocol number mapping 
    //from /etc/protocols file
    struct protoent *proto_entry;
    proto_entry = getprotobyname("tcp");
    

    //My address information to bind to
    struct sockaddr_in server;
    memset(&server, 0, sizeof(struct sockaddr));
    server.sin_family = AF_INET;
    server.sin_port = htons(7);
    
    rc= inet_pton(AF_INET, argv[1], &server.sin_addr);
    if(rc == -1){
        handle_error("Address Error");
    }
    time starttime, endtime;
    //accept incoming connections
    int addr_size = sizeof(struct sockaddr);
    int i;
    int protonum = proto_entry->p_proto;
    
    //Open an socket end-point for this TCP Server
    if((socket_fd = socket(AF_INET, SOCK_STREAM, protonum))<0){
        handle_error("Socket Error");
    }
    if(connect(socket_fd, (struct sockaddr*)&server, sizeof(server))<0){
        handle_error("Connection Error");
    }
    
    for(i=1;i<=BACKLOG;i++){
        //increase the sent message in strides of 64
        //look for for peak rtt
        msg = generate_string(STRIDE_SIZE * i);
        //printf("String Generated: \"%s\" size:%lu\n",msg, strlen(msg));
        getTime(starttime);
        if(send(socket_fd, msg, strlen(msg), 0) <0){
            handle_error("Send Error");
        }
        //The recv buffer size is same as the one we sent
        if(recv(socket_fd, buff, strlen(msg), 0) != strlen(msg)){
            handle_error("Data over/ Receive Error");
        }
        getTime(endtime);
        delta+=(diff(endtime, starttime));
        printf("Single RTT %f\n", diff(endtime, starttime));
        
    }
    
    close(socket_fd);
    printf("Averaged RTT =%f us\n", delta/BACKLOG);
    return 0;
}
   
 
