/* 
 * cswitch.c --
 *
 *	Simple program to test context switching by sending short
 *	messages back and forth through a pipe.  Invocation:
 *	
 *	cswitch count
 *
 *	Count tells how many times a one-byte message should be sent
 *	back and forth between two processes.
 *
 * Copyright 1987, 1989 Regents of the University of California
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

int func(void *arg) {
	return 0;
}

#define STACK_SIZE 4096


int main(int argc, char **argv)
{
    int count, i, toSlave[2], fromSlave[2];
    int pid;
    char buffer[10];
    unsigned int begin ,end;
    double timePer;
	unsigned int regr;
	void *child_stack = malloc(STACK_SIZE);
    int thread_pid;
   	asm volatile("mrc p15, 0, %0, c9, c14, 0" : "=r" (regr));
	printf("regr: %x\n", regr);

	count = 10;
    pipe(toSlave);
    pipe(fromSlave);
    //pid = fork();
	pid = clone(&func, child_stack+STACK_SIZE, CLONE_SIGHAND|CLONE_FS|CLONE_VM|CLONE_FILES, NULL);
	if (pid == 0) {
		write(fromSlave[1], "a", 1);
		while (1) {
			read(toSlave[0], buffer, 1);
			asm volatile("mrc p15, 0, %0, c9, c13, 0" : "=r" (end));
			if (buffer[0] != 'a') {
				printf("Exiting\n");
				return -1;
			}
	    	//write(fromSlave[1], "a", 1);
			printf("end: %u\n", end);
		}
    }
	if (pid == -1) {
		printf("Couldn't fork slave process: error %d.\n");
		return;
	}
    read(fromSlave[0], buffer, 1);
    for (i = 0; i < count; i++) {
		asm volatile("mrc p15, 0, %0, c9, c13, 0" : "=r" (begin));
		write(toSlave[1], "a", 1);
		//read(fromSlave[0], buffer, 1);
		usleep(10000);
		printf("Start: %u\n", begin);
    }
    write(toSlave[1], "b", 1);

	timePer = (end - begin)/count;
    printf("Elapsed time per ping (2 context switches): %.2f milliseconds.\n",
	    timePer);
}
